﻿using Borepin.Model;
using Borepin.Service;
using Prism.Mvvm;
using Prism.Navigation;
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;
using Xamarin.Forms;

namespace Borepin.PageModel
{
    public class MachinePageModel : BindableBase, INavigationAware
    {
        private INavigationService _NavigationService;
        private BFFHInterface _BFFHInterface;

        public MachinePageModel(INavigationService navigationService)
        {
            _NavigationService = navigationService;
            _BFFHInterface = BFFHActiveInterface.Interface;

            ReserveMachineCommand = new Command(ReserveMachineCommandExecuted);
            UseMachineCommand = new Command(UseMachineCommandExecuted);
            GiveBackMachineCommand = new Command(GiveBackMachineCommandExecuted);
            BlockMachineCommand = new Command(BlockMachineCommandExecuted);
            DisableMachineCommand = new Command(DisableMachineCommandExecuted);
        }

        private Machine _Machine;
        public Machine Machine
        {
            get => _Machine;
            set => SetProperty(ref _Machine, value);
        }

        private bool _IsValid = false;
        public bool IsValid
        {
            get => _IsValid;
            set => SetProperty(ref _IsValid, value);
        }

        private ICommand _ReserveMachineCommand;
        public ICommand ReserveMachineCommand
        {
            get => _ReserveMachineCommand;
            set => SetProperty(ref _ReserveMachineCommand, value);
        }

        private void ReserveMachineCommandExecuted()
        {
            _BFFHInterface.ReserveMachine(Machine.ID);
            UpdateMachine();
        }


        private ICommand _UseMachineCommand;
        public ICommand UseMachineCommand
        {
            get => _UseMachineCommand;
            set => SetProperty(ref _UseMachineCommand, value);
        }

        private void UseMachineCommandExecuted()
        {
            _BFFHInterface.UseMachine(Machine.ID);
            UpdateMachine();
        }

        private ICommand _GiveBackMachineCommand;
        public ICommand GiveBackMachineCommand
        {
            get => _GiveBackMachineCommand;
            set => SetProperty(ref _GiveBackMachineCommand, value);
        }

        private void GiveBackMachineCommandExecuted()
        {
            _BFFHInterface.GiveBackMachine(Machine.ID);
            UpdateMachine();
        }

        private ICommand _DisableMachineCommand;
        public ICommand DisableMachineCommand
        {
            get => _DisableMachineCommand;
            set => SetProperty(ref _DisableMachineCommand, value);
        }

        private void DisableMachineCommandExecuted()
        {
            if(Machine.State == MachineStates.Disabled)
            {
                _BFFHInterface.SetStateForce(Machine.ID, MachineStates.Free);
            }
            else
            {
                _BFFHInterface.SetStateForce(Machine.ID, MachineStates.Disabled);
            }

            UpdateMachine();
        }

        private ICommand _BlockMachineCommand;
        public ICommand BlockMachineCommand
        {
            get => _BlockMachineCommand;
            set => SetProperty(ref _BlockMachineCommand, value);
        }

        private void BlockMachineCommandExecuted()
        {
            if (Machine.State == MachineStates.Blocked)
            {
                _BFFHInterface.SetStateForce(Machine.ID, MachineStates.Free);
            }
            else
            {
                _BFFHInterface.SetStateForce(Machine.ID, MachineStates.Blocked);
            }

            UpdateMachine();
        }

        public bool CanUse
        {
            get
            {
                if(Machine == null)
                {
                    return false;
                }

                if(Machine.State == MachineStates.Free)
                {
                    return true;
                }

                if(Machine.State == MachineStates.Reserved)
                {
                    if(Machine.User == _BFFHInterface.ActiveUser)
                    {
                        return true;
                    }
                    else if(_BFFHInterface.IsAdmin())
                    {
                        return true;
                    }
                }

                return false;
            }
        }

        public bool CanReserve
        {
            get
            {
                if (Machine == null)
                {
                    return false;
                }

                if (Machine.State == MachineStates.Free)
                {
                    return true;
                }

                return false;
            }
        }

        public bool CanGiveBack
        {
            get
            {
                if (Machine == null)
                {
                    return false;
                }

                if (Machine.State == MachineStates.InUse && Machine.User == _BFFHInterface.ActiveUser)
                {
                    return true;
                }

                return false;
            }
        }

        public bool IsAdmin
        {
            get
            {
                if (_BFFHInterface == null)
                {
                    return false;
                }

                if (_BFFHInterface.IsAdmin())
                {
                    return true;
                }

                return false;
            }
        }

        public void UpdateMachine()
        {
            Machine = BFFHService.GetMachine(Machine.ID);

            OnPropertyChanged(new System.ComponentModel.PropertyChangedEventArgs("CanUse"));
            OnPropertyChanged(new System.ComponentModel.PropertyChangedEventArgs("CanReserve"));
            OnPropertyChanged(new System.ComponentModel.PropertyChangedEventArgs("CanGiveBack"));
        }

        public void OnNavigatedFrom(INavigationParameters parameters)
        {
            
        }

        public void OnNavigatedTo(INavigationParameters parameters)
        {
            string machineID = parameters["machineID"] as string;

            if(machineID == null)
            {
                _IsValid = false;
                return;
            }

            try
            {
                Machine = BFFHService.GetMachine(machineID);
                IsValid = true;

                UpdateMachine();
            }
            catch
            {
                _IsValid = false;
            }
        }
    }
}
